/*
 * UartContext.cpp
 *
 *  Created on: Sep 5, 2017
 *      Author: guoxs
 */
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <memory.h>
#include <termio.h>
#include <sys/ioctl.h>

#include "uart/ModbusContext.h"
#include "uart/ProtocolData.h"
#include "uart/ProtocolParser.h"
#include "utils/Log.h"

#include "modbus/modbus.h"

#define UART_DATA_BUF_LEN			16384	// 16KB

extern void parseinputRegProtocol(uint16_t *pData, UINT len);
extern void parseRegProtocol(uint16_t addr,uint16_t *pData, UINT len);

typedef struct{
	uint16_t addr;
	uint16_t data;
}S_MODBUS_DATA;

typedef struct{
	uint16_t addr;
	uint16_t data;
}S_MODBUS_COIL;

typedef struct{
	uint16_t addr;
	uint16_t datas[64];
	BYTE datalen;
}S_MODBUS_DATAS;

std::vector<S_MODBUS_DATA> modbusSend;
std::vector<S_MODBUS_COIL> modbusCoilSend;
std::vector<S_MODBUS_DATAS> modbusSendDatas;
ModbusContext::ModbusContext() :
	mIsOpen(false),
	mUartID(0),
	mDataBufPtr(NULL),
	mDataBufLen(0) {
	modbusSend.clear();
	modbusCoilSend.clear();
	modbusSendDatas.clear();
}

ModbusContext::~ModbusContext() {
	delete[] mDataBufPtr;
	closeUart();
}
static modbus_t* mctx;
bool ModbusContext::openUart(const char *pFileName, UINT baudRate) {
	LOGD("openUart pFileName = %s, baudRate = %d\n", pFileName, baudRate);

	mctx = modbus_new_rtu(pFileName, baudRate, 'N', 8, 1);

	if (mctx == 0) {
		mIsOpen = false;
	} else {
		modbus_set_slave(mctx, 1);
		modbus_connect(mctx);
		//modbus_set_byte_timeout(mctx, 5,0);
		modbus_set_response_timeout(mctx, 1,0);
		modbus_set_indication_timeout(mctx, 1,0);
		modbus_set_debug(mctx,1);

		mIsOpen = run("uart");
		if (!mIsOpen) {
		}

		LOGD("openUart mIsOpen = %d\n", mIsOpen);
	}

	return mIsOpen;
}

void ModbusContext::closeUart() {
	LOGD("closeUart mIsOpen: %d...\n", mIsOpen);
	if (mIsOpen) {
		requestExit();
		modbus_close(mctx);
		mctx = 0;
		mIsOpen = false;
	}
}


ModbusContext* ModbusContext::getInstance() {
	static ModbusContext sUC;
	return &sUC;
}

bool ModbusContext::readyToRun() {
	if (mDataBufPtr == NULL) {
		mDataBufPtr = new BYTE[UART_DATA_BUF_LEN];
	}

	if (mDataBufPtr == NULL) {
		closeUart();
	}

	return (mDataBufPtr != NULL);
}
extern void parseBitsProtocol(uint16_t addr,uint8_t *pData, UINT len);
bool ModbusContext::threadLoop() {
	uint16_t dest[20];
	uint8_t bits[20];
 	static int selftestStep = -1;
	int ret;
	if (mIsOpen) {
		/**
		 * 在这里按照自己自己的需要读取需要读取的寄存器
		 *
		 */
		//read coils,01 cmd
		ret = modbus_read_bits(mctx,PLC_COILS_BASEADDR+0x04,1,bits);
		//LOGD("read reg req:%d",ret);
		if(ret == -1){
		}else{
			parseBitsProtocol(PLC_COILS_BASEADDR+0x04,bits,1);
		}

		//19-22
		ret = modbus_read_registers(mctx,PLC_REGISTOR_BASEADDR+19, 8, dest);
		if(ret != -1){
			parseRegProtocol(PLC_REGISTOR_BASEADDR+19,dest,8);
		}

		processSendCmd();
		sleep(100); // sleep 100ms

		return true;
	}

	return false;
}

bool ModbusContext::send(const BYTE* pData, UINT len) {
}

bool ModbusContext::writeRegister(uint16_t addr, uint16_t data) {
	LOGD("modbus write:%d[%d]",addr,data);
	S_MODBUS_DATA modbusdata;
	modbusdata.addr = addr;
	modbusdata.data = data;
	modbusSend.push_back(modbusdata);
	return 1;
}

bool ModbusContext::writeRegisters(uint16_t addr, uint16_t* data, BYTE len) {
	LOGD("modbus write datas :%d[%d]",addr,len);
	S_MODBUS_DATAS modbusdata;
	modbusdata.addr = addr;
	if(len > 64) return 0;
	memcpy(modbusdata.datas,data,len*sizeof(uint16_t));
	modbusdata.datalen = len;
	modbusSendDatas.push_back(modbusdata);
	return 1;
}

void ModbusContext::processSendCmd() {
	int ret;
	int retry;
	if(modbusSend.size() > 0){
		for(int i =0;i < modbusSend.size();i++){
			S_MODBUS_DATA modbusdata;
			modbusdata = modbusSend.at(i);
			for(retry = 0;retry < 3;retry++){
				ret = modbus_write_register(mctx, modbusdata.addr, modbusdata.data);
				if(ret != -1) break;
			}
		}
		modbusSend.clear();
	}
	if(modbusSendDatas.size() > 0){
		for(int i =0;i < modbusSendDatas.size();i++){
			S_MODBUS_DATAS modbusdata;
			modbusdata = modbusSendDatas.at(i);
			for(retry = 0;retry < 3;retry++){
				ret = modbus_write_registers(mctx, modbusdata.addr,modbusdata.datalen, modbusdata.datas);
				if(ret != -1) break;
			}
		}
		modbusSendDatas.clear();
	}
	if(modbusCoilSend.size() > 0){
		for(int i =0;i < modbusCoilSend.size();i++){
			S_MODBUS_COIL modbusdata;
			modbusdata = modbusCoilSend.at(i);
			for(retry = 0;retry < 3;retry++){
				ret = modbus_write_bit(mctx, modbusdata.addr, modbusdata.data);
				if(ret != -1) break;
			}
		}
		modbusCoilSend.clear();
	}



}
/**
 * 按照位写入
 */
bool ModbusContext::writeCoil(uint16_t addr, BYTE data) {
	LOGD("modbus write coil:%d[%d]",addr,data);
	S_MODBUS_COIL modbusdata;
	modbusdata.addr = addr;
	modbusdata.data = data;
	modbusCoilSend.push_back(modbusdata);
	return 1;
}

bool ModbusContext::setAddress(uint8_t addr) {
	return modbus_set_slave(mctx, addr);
}
