/*
 * ProtocolParser.cpp
 *
 *  Created on: Sep 7, 2017
 *      Author: guoxs
 */
#include <vector>
#include <system/Mutex.h>
#include "CommDef.h"
#include "uart/ProtocolParser.h"
#include "utils/Log.h"

static Mutex sLock;
static std::vector<OnProtocolDataUpdateFun> sProtocolDataUpdateListenerList;

void registerProtocolDataUpdateListener(OnProtocolDataUpdateFun pListener) {
	Mutex::Autolock _l(sLock);
	LOGD("registerProtocolDataUpdateListener\n");
	if (pListener != NULL) {
		sProtocolDataUpdateListenerList.push_back(pListener);
	}
}

void unregisterProtocolDataUpdateListener(OnProtocolDataUpdateFun pListener) {
	Mutex::Autolock _l(sLock);
	LOGD("unregisterProtocolDataUpdateListener\n");
	if (pListener != NULL) {
		std::vector<OnProtocolDataUpdateFun>::iterator iter = sProtocolDataUpdateListenerList.begin();
		for (; iter != sProtocolDataUpdateListenerList.end(); iter++) {
			if ((*iter) == pListener) {
				sProtocolDataUpdateListenerList.erase(iter);
				return;
			}
		}
	}
}

static void notifyProtocolDataUpdate(const SProtocolData &data) {
	Mutex::Autolock _l(sLock);
	std::vector<OnProtocolDataUpdateFun>::const_iterator iter = sProtocolDataUpdateListenerList.begin();
	for (; iter != sProtocolDataUpdateListenerList.end(); iter++) {
		(*iter)(data);
	}
}

static SProtocolData sProtocolData;

SProtocolData& getProtocolData(){
	return sProtocolData;
}

/**
 * 解析每一帧数据
 */
void parseinputRegProtocol(UINT16 addr,UINT16 *pData, UINT len) {


	// 通知协议数据更新
	notifyProtocolDataUpdate(sProtocolData);
}

void parseRegProtocol(uint16_t addr,uint16_t *pData, UINT len){
	if((addr >= PLC_REGISTOR_BASEADDR) && (addr <= PLC_REGISTOR_BASEADDR+PLC_REGISTOR_LEN)){
		memcpy(&sProtocolData.regdata[addr-PLC_REGISTOR_BASEADDR],pData,len*sizeof(uint16_t));
	}
	if((addr >= PLC_FDATA_BASEADDR) && (addr <= PLC_FDATA_BASEADDR+PLC_FDATA_LEN)){
		memcpy(&sProtocolData.fdData[addr-PLC_FDATA_BASEADDR],pData,len*sizeof(uint16_t));
	}
	// 通知协议数据更新
	notifyProtocolDataUpdate(sProtocolData);
}

/**
 * 解析每一帧数据
 */
void parseBitsProtocol(uint16_t addr,uint8_t *pData, UINT len) {

	if((addr >= PLC_COILS_BASEADDR) && (addr <= PLC_COILS_BASEADDR+PLC_COILS_LEN)){
		memcpy(&sProtocolData.coils[addr-PLC_COILS_BASEADDR],pData,len);
	}
	// 通知协议数据更新
	notifyProtocolDataUpdate(sProtocolData);
}
